program MDI_Ribbon;

{$R MDI_RIB.RES}

uses
  BWCC,
  WinTypes,
  WinProcs,
  Strings,
  Objects,
  OWindows,
  ODialogs,
  OStdDlgs,
  Win31,
  WinCrt,

  Ribbon,
  Tools;

const
  cm_NrOfChildren  = 101;

  cm_RibbonShow    = 201;
  cm_RibbonHide    = 202;
  cm_RibbonEdit    = 203;

  cm_HelpIndex     = 998;
  cm_HelpAbout     = 999;

  id_CantClose     = 500;

  { Although one expects to be able to use wm_MDISetMenu to handle the
    (re)setting of menu-item states, it turns out that you are better of
    using the message below. This is because the wm_MDISetMenu is handled
    during destruction of a MDI-childwindow. The message below is
    send after destruction of a MDI-childwindow.

    You may find it interesting using WinSight to explore this message...}
  wm_SetMenu       = wm_User + $0CAC;

type
  TMDIApp = object(TApplication)
    procedure InitMainWindow; virtual;
  end;

  PMyMDIChild = ^TMyMDIChild;
  TMyMDIChild = object(TWindow)
    Num: Integer;
    constructor Init(AParent: PWindowsObject; ChildNum: Integer);
  end;

  PMyClientWindow = ^TMyClientWindow;
  TMyClientWindow = object( TMDIClient )
    function    NrOfChildren: integer;
    procedure   SetMenuItems;
    procedure   SetItem( aCmd, aState: word );
    procedure   WMSetMenu( var Msg: TMessage );
      virtual wm_First + wm_SetMenu;
  end;

  PMyMDIWindow = ^TMyMDIWindow;
  TMyMDIWindow = object(TMDIWindow)
    Ribbon     : PRibbon;

    constructor Init( aTitle: PChar; aMenu: HMenu );
    destructor  Done; virtual;
    procedure   InitClientWindow; virtual;
    procedure   SetupWindow; virtual;
    procedure   LoadPrefs;
    procedure   SavePrefs;
    function    CreateChild: PWindowsObject; virtual;

    procedure   WMSize( var Msg: TMessage );
      virtual wm_First + wm_Size;

    procedure   CMMDIFileNew( var Msg: TMessage );
      virtual cm_First + cm_MDIFileNew;
    procedure   CMMDIFileOpen( var Msg: TMessage );
      virtual cm_First + cm_MDIFileOpen;
    procedure   CMFileSave( var Msg: TMessage );
      virtual cm_First + cm_FileSave;
    procedure   CMNrOfChildren( var Msg: TMessage );
      virtual cm_First + cm_NrOfChildren;

    procedure   CMEditCut( var Msg: TMessage );
      virtual cm_First + cm_EditCut;
    procedure   CMEditCopy( var Msg: TMessage );
      virtual cm_First + cm_EditCopy;
    procedure   CMEditPaste( var Msg: TMessage );
      virtual cm_First + cm_EditPaste;

    procedure   CMRibbonShow( var Msg: TMessage );
      virtual cm_First + cm_RibbonShow;
    procedure   CMRibbonHide( var Msg: TMessage );
      virtual cm_First + cm_RibbonHide;
    procedure   CMRibbonEdit( var Msg: TMessage );
      virtual cm_First + cm_RibbonEdit;

    procedure   CMHelpIndex( var Msg: TMessage );
      virtual cm_First + cm_HelpIndex;
    procedure   CMHelpAbout( var Msg: TMessage );
      virtual cm_First + cm_HelpAbout;
  end;

{=========================================================================}
{== Implementation of TMyMDIChild                                       ==}
{=========================================================================}

constructor TMyMDIChild.Init(AParent: PWindowsObject; ChildNum: Integer);
var
  TitleStr: array[0..12] of Char;
  ChildNumStr: array[0..5] of Char;
begin
  Str(ChildNum, ChildNumStr);
  StrCat(StrECopy(TitleStr, 'Child #'), ChildNumStr);
  inherited Init(AParent, TitleStr);
  Num := ChildNum;
end;

{=========================================================================}
{== Implementation of TMyClientWindow                                   ==}
{=========================================================================}

function TMyClientWindow.NrOfChildren: integer;

var
  I: integer;

  procedure FindChild( P: PWindowsObject ); far;
  begin
    if StrComp( P^.GetClassName, 'TurboWindow' ) = 0 then
      Inc( I );
  end;

begin
  I := 0;
  Parent^.ForEach(@FindChild);
  NrOfChildren := I;
end;

{=========================================================================}

procedure TMyClientWindow.SetMenuItems;

begin
  if NrOfChildren > 0 then
    begin
      SetItem( cm_FileSave, mf_Enabled );
      SetItem( cm_EditCut, mf_Enabled );
      SetItem( cm_EditCopy, mf_Enabled );
      SetItem( cm_EditPaste, mf_Enabled );
    end
  else
    begin
      SetItem( cm_FileSave, mf_Grayed );
      SetItem( cm_EditCut, mf_Grayed );
      SetItem( cm_EditCopy, mf_Grayed );
      SetItem( cm_EditPaste, mf_Grayed );
    end;
end;

{=========================================================================}

procedure TMyClientWindow.SetItem( aCmd, aState: word );

begin
  EnableMenuItem( PMyMDIWindow( Parent )^.Attr.Menu, aCmd,
                  mf_ByCommand or aState );

  { This is how you change the state of a button. It does not matter if
    the button exists. That way you can use 'SetItem' for all your
    state-settings }
  PMyMDIWindow( Parent )^.Ribbon^.EnableBtn( aCmd, aState );
end;

{=========================================================================}

procedure TMyClientWindow.WMSetMenu( var Msg: TMessage );

begin
  SetMenuItems;
  DefWndProc( Msg );
end;

{=========================================================================}
{== Implementation of TMyMDIWindow                                      ==}
{=========================================================================}

constructor TMyMDIWindow.Init( aTitle: PChar; aMenu: HMenu );

begin
  inherited Init( aTitle, aMenu );

  ChildMenuPos := 3; { Children under menu 'Window' }

  { Here is where we define our ribbon. By specifying 'rbn_EnableEdit' you
    will be able to edit the ribbon-contents. If you do not want your
    users to edit the ribbon, just use 'rbn_DisableEdit' }
  Ribbon := New( PRibbon, Init( @Self, rbn_EnableEdit ));

  with Ribbon^ do
    begin
      { At this place we define all available buttons. We do this by
        calling AddButton with the following parameters:
        - BtnID :  This is the ID under which you define your bitmaps
                   in the Resource Workshop, for example:
                     use 101 for your images 1101 (btn-up) and
                     3101 (btn-down).
        - CmdID :  This is the command-ID you specified for your menu-
                   item. It will be send to the mainwindow when you
                   press a ribbon-button.
        - Descr :  Just a short description that will be shown in
                   the 'Edit_Ribbon' dialog.
        - Flag  :  TRUE if the button has to be active at startup,
                   otherwise false.
      }
      AddButton( 208, cm_HelpIndex, 'Display help index', TRUE );
      { The simplest way to add a separator...}
      AddSeparator;
      AddButton( 201, cm_MDIFileNew, 'Create a new file', TRUE );
      AddButton( 202, cm_MDIFileOpen, 'Open an existing file', TRUE );
      AddButton( 203, cm_FileSave, 'Save active document', TRUE );
      AddSeparator;
      AddButton( 205, cm_EditCut, 'Cut selection to clipboard', FALSE );
      AddButton( 206, cm_EditCopy, 'Copy selection to clipboard', FALSE );
      AddButton( 207, cm_EditPaste, 'Paste clipboard to document', FALSE );
    end;{ with }

  if FileExist( 'MDI_RIB.PRF' ) then
    LoadPrefs;
end;

{=========================================================================}

destructor TMyMDIWindow.Done;

begin
  SavePrefs;
  inherited Done;
end;

{=========================================================================}

procedure TMyMDIWindow.InitClientWindow;

begin
  ClientWnd := New( PMyClientWindow, Init( @Self ));
end;

{=========================================================================}

procedure TMyMDIWindow.SetupWindow;

begin
  inherited SetupWindow;
  { ...and just show them... }
  Ribbon^.ShowButtons;
  SendMessage( ClientWnd^.HWindow, wm_MDISetMenu, word(TRUE), 0 );
end;

{=========================================================================}

procedure TMyMDIWindow.LoadPrefs;

var
  OpenFile         : TDosStream;

begin
  { And this is an example of loading the last state of the ribbon }
  OpenFile.Init( 'MDI_RIB.PRF', stOpen );
  Ribbon^.AllButtons := PCollection( OpenFile.Get );
  OpenFile.Done;
end;

{=========================================================================}

procedure TMyMDIWindow.SavePrefs;

var
  SaveFile         : TDosStream;

begin
  SaveFile.Init( 'MDI_RIB.PRF', stCreate );
  { As you can see, the buttons are kept in a PCollection }
  SaveFile.Put( Ribbon^.AllButtons );
  SaveFile.Done;
end;

{=========================================================================}

function TMyMDIWindow.CreateChild: PWindowsObject;
var
  ChildNum: Integer;

  function NumberUsed(P: PMyMDIChild): Boolean; far;
  begin
    NumberUsed := ChildNum = P^.Num;
  end;

begin
  ChildNum := 1;
  while FirstThat(@NumberUsed) <> nil do Inc(ChildNum);
  CreateChild := Application^.MakeWindow(New(PMyMDIChild,
    Init(@Self, ChildNum)));
end;

{=========================================================================}

procedure TMyMDIWindow.WMSize(var Msg: TMessage);

var
  rcUpdate         : TRect;

begin
 { sendmessage(HWindow,wm_setredraw,0,0); }
  inherited WMSize(Msg);

  { Always check for nil window pointers and invalid HWindow handles.
    Windows can send your main window WMSize messages while the child
    windows are being destroyed, which can produce some hard to track
    down UAEs.  }

    if (ClientWnd <> nil) and (ClientWnd^.HWindow <> 0) then
      MoveWindow( ClientWnd^.HWindow, 0, Ribbon^.Height, Msg.LParamLo,
                  Msg.LParamHi - Ribbon^.Height, True);

    if (Ribbon <> nil) then
      Ribbon^.ReSize( Msg );

{  sendmessage(HWindow,wm_setredraw,1,0);
  RedrawWindow(Hwindow,@rcUpdate,0,
	 RDW_INVALIDATE or RDW_UPDATENOW or RDW_ALLCHILDREN); }
end;

{=========================================================================}

procedure TMyMDIWindow.CMMDIFileNew( var Msg: TMessage );

begin
  CreateChild;
end;

{=========================================================================}

procedure TMyMDIWindow.CMMDIFileOpen( var Msg: TMessage );
var
  aFileName        : array[0..79] of char;
  FileDlg          : PFileDialog;

begin
  StrCopy( aFileName, '*.*');
  FileDlg :=  New( PFileDialog, Init( @Self, PChar( sd_FileOpen ), aFileName ));
  Application^.ExecDialog( FileDlg );
  BWCCMessageBox( HWindow, 'This was just a dummy file open action',
                  'Message', mb_OK or mb_IconInformation );
end;

{=========================================================================}

procedure TMyMDIWindow.CMFileSave( var Msg: TMessage );

begin
  BWCCMessageBox( HWindow, 'You could save the file here...',
                  'Message', mb_OK or mb_IconInformation );
end;

{=========================================================================}

procedure TMyMDIWindow.CMNrOfChildren( var Msg: TMessage );

var
  Message          : array[0..79] of char;
  Value            : integer;
  ValStr           : array[0..10] of char;

begin
  Value := PMyClientWindow( ClientWnd )^.NrOfChildren;
  Str( Value, ValStr );
  BWCCMessageBox( HWindow, ValStr, 'Number of child windows', mb_OK );
end;

{=========================================================================}

procedure TMyMDIWindow.CMEditCut( var Msg: TMessage );

begin
  BWCCMessageBox( HWindow, 'Cutting selection to clipboard...',
                  'Message', mb_OK or mb_IconInformation );
end;

{=========================================================================}

procedure TMyMDIWindow.CMEditCopy( var Msg: TMessage );

begin
  BWCCMessageBox( HWindow, 'Copying selection to clipboard...',
                  'Message', mb_OK or mb_IconInformation );
end;

{=========================================================================}

procedure TMyMDIWindow.CMEditPaste( var Msg: TMessage );

begin
  BWCCMessageBox( HWindow, 'Pasting from clipboard...',
                  'Message', mb_OK or mb_IconInformation );
end;

{=========================================================================}

procedure TMyMDIWindow.CMRibbonShow( var Msg: TMessage );

begin
  Ribbon^.Visible;
end;

{=========================================================================}

procedure TMyMDIWindow.CMRibbonHide( var Msg: TMessage );

begin
  Ribbon^.Hide;
end;

{=========================================================================}

procedure TMyMDIWindow.CMRibbonEdit( var Msg: TMessage );

begin
  Ribbon^.Edit;
end;

{=========================================================================}

procedure TMyMDIWindow.CMHelpIndex( var Msg: TMessage );

begin
  BWCCMessageBox( HWindow, 'Open your Help-file index...',
                  'Message', mb_OK or mb_IconInformation );
end;

{=========================================================================}

procedure TMyMDIWindow.CMHelpAbout( var Msg: TMessage );

var
  Dlg              : PDialog;

begin
  Dlg := New( PDialog, Init( @Self, 'About' ));
  Application^.ExecDialog( Dlg );
end;

{=========================================================================}
{== Implementation of TMDIApp                                           ==}
{=========================================================================}

procedure TMDIApp.InitMainWindow;
begin
  MainWindow := New(PMyMDIWindow,
    Init('The ribbon in a MDI-application', LoadMenu(HInstance, 'MainMenu')));
end;

var
  MDIApp: TMDIApp;
  Screen:text;

begin
  assigncrt(screen);
  MDIApp.Init('MDITest');
  MDIApp.Run;
  MDIApp.Done;
end.
